@@ -191,12 +191,25 @@ class Subkernels(object):
                         color=matched[1],
                         options=matched[2]))
             else:
-                # undefined language also use default theme color
-                self._kernel_list.append(
-                    subkernel(
-                        name=spec,
-                        kernel=spec,
-                        language=km.get_kernel_spec(spec).language))
+                lan_name = km.get_kernel_spec(spec).language
+                if lan_name == 'python':
+                    lan_name = 'python3'
+                avail_names = [x for x in lan_map.keys() if x.lower() == lan_name.lower()]
+                if avail_names:
+                    self._kernel_list.append(
+                        subkernel(
+                            name=spec,
+                            kernel=spec,
+                            language=lan_map[avail_names[0]][0],
+                            color=lan_map[avail_names[0]][1],
+                            options=lan_map[avail_names[0]][2]))
+                else:
+                    # undefined language also use default theme color
+                    self._kernel_list.append(
+                        subkernel(
+                            name=spec,
+                            kernel=spec,
+                            language=lan_name))
 
     def kernel_list(self):
         return self._kernel_list
@@ -882,12 +895,14 @@ class SoS_Kernel(IPythonKernel):
 
     def get_vars_from(self, items, from_kernel=None, explicit=False):
         if from_kernel is None or from_kernel.lower() == 'sos':
+            # Feature removed #253
             # autmatically get all variables with names start with 'sos'
-            default_items = [
-                x for x in env.sos_dict.keys()
-                if x.startswith('sos') and x not in self.original_keys
-            ]
-            items = default_items if not items else items + default_items
+            # default_items = [
+            #     x for x in env.sos_dict.keys()
+            #     if x.startswith('sos') and x not in self.original_keys
+            # ]
+            if items is None:
+                items = []
             for item in items:
                 if item not in env.sos_dict:
                     self.warn(f'Variable {item} does not exist')
@@ -989,6 +1004,10 @@ class SoS_Kernel(IPythonKernel):
             if isinstance(objects, dict):
                 # returns a SOS dictionary
                 try:
+                    # if the variable is passing through SoS, let us try to restore variables in SoS
+                    if to_kernel is not None:
+                        missing_vars = [x for x in objects.keys() if x not in env.sos_dict]
+                        existing_vars = {x:env.sos_dict[x] for x in objects.keys() if x in env.sos_dict}
                     env.sos_dict.update(objects)
                 except Exception as e:
                     self.warn(
@@ -1009,6 +1028,10 @@ class SoS_Kernel(IPythonKernel):
                 finally:
                     # switch back to the original kernel
                     self.switch_kernel(my_kernel)
+                    # restore sos_dict to avoid bypassing effect #252
+                    for missing_var in missing_vars:
+                        env.sos_dict.pop(missing_var)
+                    env.sos_dict.update(existing_vars)
             elif isinstance(objects, str):
                 # an statement that will be executed in the destination kernel
                 if to_kernel is None or to_kernel == 'SoS':
